#!/usr/bin/env bash

# The 'get_a_digit' function uses the $RANDOM function in Bash shell to generate a random number from 0-32767 and then captures the least significant digit that is between 1-6... simulating a single dice roll. The digit is stored in the digit.txt file.

# The 'get_a_word' function repeats 'get_a_digit' to create a 5-digit number to match against the Diceware(TM) Word List, dicelist.txt. The UNTIL loop is the trap to ensure that a valid digit has been created. Earlier versions would error because $RANDOM would sometimes create a number containing only 7, 8, 9, and 0, and then the TR command would strip them all away and leave an empty value being passed to digit.txt. After trapping for a valid digit, each one is appended to passphrase.txt to create the 5-digit total needed.

# The 'finish' function GREPS the created 5-digit group in passphrase.txt for a word match in dicelist.txt, and the matching results are appended back into the passphrase.txt file.

# The script allows a user to select a 6-word, 8-word, or 10-word passphrase. It then uses a WHILE loop to repeatedly execute 'get_a word' and 'finish' functions until the user's selection is completed and appended to passphrase.txt. The results are displayed in the terminal, and saved in passphrase.txt. The passphrase.txt file can be easily made to self-delete by uncommenting the 'rm passphrase.txt' line near the end of the script and saving the script file.

# Diceware(TM) is a trademark of Arnold G. Reinhold. See more at http://www.diceware.com.

# WARNING! The Diceware(TM) author strongly discourages the use of electronic number generators such as this script, as the quality of the random number cannot be certain. He is right, of course. Do not use this script if you have any concerns about the security of the passphrase it generates. If you have extremely high security needs, use real dice. For my use, the script seems to be sufficiently random enough. The creation of this script was just for fun and learning.

# Dice version 1.0 completed 12 March 2018 by Stan Vandiver http://www.linuxgeeks.us. Copyright (C) 2017-2018

# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details <http://www.gnu.org/licenses/>.


# FUNCTIONS

function get_a_digit {
    echo $RANDOM | tr -d "[0,7,8,9]" | awk '{printf substr($0,length,1)}' > digit.txt
}


function get_a_word {

    get_a_digit #1
    until [[ -s digit.txt ]]
    do
        get_a_digit
    done
    cat digit.txt >> passphrase.txt


    get_a_digit #2
    until [[ -s digit.txt ]]
    do
        get_a_digit
    done
    cat digit.txt >> passphrase.txt


    get_a_digit #3
    until [[ -s digit.txt ]]
    do
        get_a_digit
    done
    cat digit.txt >> passphrase.txt


    get_a_digit #4
    until [[ -s digit.txt ]]
    do
        get_a_digit
    done
    cat digit.txt >> passphrase.txt


    get_a_digit #5
    until [[ -s digit.txt ]]
    do
        get_a_digit
    done
    cat digit.txt >> passphrase.txt
}


function finish {
    grep -f passphrase.txt dicelist.txt >> passphrase.txt
}

# END OF FUNCTIONS


# BEGINNING OF SCRIPT

clear

printf "\nPlease visit diceware.com for more info about passphrase security.\n"
printf "\nWould you like a 6-word passphrase? 8-words? 10-words? (Enter 6, 8, or 10): "
read number_of_words
printf "\n"

printf "Dice\tWord list Match\n" > passphrase.txt


count=0

if [ $number_of_words -eq 6 ]
then
    while [ $count -le 5 ];
    do get_a_word && finish
    (( count++ ))
    done
fi

if [ $number_of_words -eq 8 ]
then
    while [ $count -le 7 ];
    do get_a_word && finish
    (( count++ ))
    done
fi

if [ $number_of_words -eq 10 ]
then
    while [ $count -le 9 ];
    do get_a_word && finish
    (( count++ ))
    done
fi


cat passphrase.txt
printf "\n"
#rm passphrase.txt

# END OF SCRIPT
# Uncomment 'rm passphrase.txt' above if you want the file to automatically delete itself after generating each passphrase.
