#!/usr/bin/env bash

# Generates random passphrases using the Diceware(TM) Word list
# The script prompts the user to enter the number of words to generate for their passphrase. 
# It repeatedly executes the 'get_word_code' function* to randomly generate a 5 digit number, and finds the corresponding word in the Diceware(TM) Word List, dicelist.txt, until the required number of words have been generated. The results are displayed in the terminal, and saved in passphrase.txt. The passphrase.txt file can be easily made to self-delete by uncommenting the 'rm passphrase.txt' line near the end of the script and saving the script file.

# Diceware(TM) is a trademark of Arnold G. Reinhold. See more at http://www.diceware.com.

# WARNING! The Diceware(TM) author strongly discourages the use of electronic number generators such as this script, as the quality of the random number cannot be certain. He is right, of course. Do not use this script if you have any concerns about the security of the passphrase it generates. If you have extremely high security needs, use real dice. For my use, the script seems to be sufficiently random enough. The creation of this script was just for fun and learning.

# Dice version 1.0 completed 12 March 2018 by Stan Vandiver http://www.linuxgeeks.us. Copyright (C) 2017-2018
# Additional edits by Jason Trunks https://notabug.org/JasKinasis

# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details <http://www.gnu.org/licenses/>.


# FUNCTIONS

# Quit with an error message
function die {
	echo >&2 "ERROR:" "$@"
	exit 1
}


# Generate a 5 digit random number based on the roll of a 6-sided dice
function get_word_code {
	digits=[]                   
    for digitcount in {0..4}; do
		digits[$digitcount]=$(( RANDOM % 6 + 1 )) 
	done
	echo -n "${digits[@]}" | sed 's/ //g' 
}

# END OF FUNCTIONS


# BEGINNING OF SCRIPT

clear
printf "\nPlease visit diceware.com for more info about passphrase security.\n"
printf "\nHow many words should the passphrase contain? "
read number_of_words
printf "\n"

# Ensure user entered a valid number
number_of_words=$(echo "${number_of_words}" | awk '/^[0-9]+$/')
[[ $number_of_words ]] || die "Invalid number entered."
[[ $number_of_words -gt 0 ]] || die "Number must be a positive integer."

printf "Dice\tWord list Match\n" > passphrase.txt

# Generate words
count=0
while [[ $count -lt "${number_of_words}" ]]
do
    \grep "$(get_word_code)" dicelist.txt | sed 's/   //g' >> passphrase.txt
	(( count++ ))
done

# Display results
cat passphrase.txt
printf "\n"
#rm passphrase.txt

# END OF SCRIPT
# Uncomment 'rm passphrase.txt' above if you want the file to automatically delete itself after generating each passphrase.
